function [retrieved_displacement, retrieved_displacement_correc, predicted_displacement, sgf] = calculate_displacements_1D(Ima_ref,Ima_def,s,dep_GT,interpscheme,case_disp)

[x,y] = meshgrid(1:size(Ima_ref,2),1:size(Ima_ref,1));
m=floor(size(Ima_ref,1)/2);

t=2*s+1;
retrieved_displacement=zeros(size(Ima_ref));
retrieved_displacement_correc=zeros(size(Ima_ref));
predicted_displacement=zeros(size(Ima_ref));

[Ima_def_derx,~]=gradient(Ima_def);

gamma = 0.9; 
p = 80;

% weight for the SSD-tilde criterion (eq. 4.35)
alpha = 128*gamma*2*pi*1.5*(x+dep_GT).^0.5/p^1.5.*cos(2*pi*(x+dep_GT).^1.5/p^1.5);
kappa=1e-4;
weight=1./max(alpha.^2,kappa);  

options=optimoptions('fminunc','Display','off','MaxIterations',10000,'OptimalityTolerance',1e-10,'StepTolerance',1e-10);

if (strcmp(interpscheme,'exact'))
    h = waitbar(0,'Initializing waitbar...','Name','SSD minimization');
    for j=m-s:m+s
        ratio=100*(j-m+s)/(2*s);
        waitbar(ratio/100,h,sprintf('%d%% along...',uint8(ratio)))  
        parfor i=(s+1):size(Ima_ref,2)-(s+1)
            f = @(phi)funexact(phi,i,j,s,case_disp);
            phi=fminunc(f,0.0,options);
            retrieved_displacement(j,i)=phi;
            predicted_displacement(j,i) = sum(sum( alpha(j-s:j+s,i-s:i+s).^2 .* dep_GT(j-s:j+s,i-s:i+s) )) /  sum(sum( alpha(j-s:j+s,i-s:i+s).^2));
        end
    end
    close(h)
    sgf = sgolayfilt(sgolayfilt(dep_GT',0,t)',0,t);
    h = waitbar(0,'Initializing waitbar...','Name','SSD tilde minimization');
    for j=m-s:m+s
        ratio=100*(j-m+s)/(2*s);
        waitbar(ratio/100,h,sprintf('%d%% along...',uint8(ratio)))  
        parfor i=(s+1):size(Ima_ref,2)-(s+1)
            phi=retrieved_displacement(j,i);
            f = @(phi)funweightexact(phi,i,j,s,weight,case_disp);
            phi=fminunc(f,phi,options);
            retrieved_displacement_correc(j,i)=phi;  
        end
    end
    close(h)
    
else
    h = waitbar(0,'Initializing waitbar...','Name','SSD minimization');
    for j=m-s:m+s
        ratio=100*(j-m+s)/(2*s);
        waitbar(ratio/100,h,sprintf('%d%% along...',uint8(ratio)))
        parfor i=(s+1):size(Ima_ref,2)-(s+1)
            f = @(phi)fun(phi,Ima_def,Ima_ref,x,y,i,j,s,interpscheme);
            phi=fminunc(f,0.0,options);
            retrieved_displacement(j,i)=phi;
            Ima_def_registered_exact = 128*(1+gamma*sin(2*pi*(x+phi).^1.5/p^1.5));
            Ima_def_registered_interp = interp2(x,y,Ima_def,x+phi,y,interpscheme); 
            DIprime = Ima_def_registered_exact - Ima_def_registered_interp;
            predicted_displacement(j,i) = sum(sum( alpha(j-s:j+s,i-s:i+s).^2 .* dep_GT(j-s:j+s,i-s:i+s) )) /  sum(sum( alpha(j-s:j+s,i-s:i+s).^2)) + ...
            sum(sum( alpha(j-s:j+s,i-s:i+s) .* DIprime(j-s:j+s,i-s:i+s) )) /  sum(sum( alpha(j-s:j+s,i-s:i+s).^2));
        end
    end
    close(h)
    sgf = sgolayfilt(sgolayfilt(dep_GT',0,t)',0,t);
    h = waitbar(0,'Initializing waitbar...','Name','SSD tilde minimization');
    for j=m-s:m+s
        ratio=100*(j-m+s)/(2*s);
        waitbar(ratio/100,h,sprintf('%d%% along...',uint8(ratio)))
        parfor i=(s+1):size(Ima_ref,2)-(s+1)
            phi=retrieved_displacement(j,i);
            f = @(phi)funweight(phi,Ima_def,Ima_ref,x,y,i,j,s,weight,Ima_def_derx,interpscheme);
            phi=fminunc(f,phi,options);
            retrieved_displacement_correc(j,i)=phi;  
        end    
    end
    close(h)

end


return


