clear all, close all
% script for reproducing the experiments in Section 4.1 of:
% "On biases in displacement estimation for image registration, 
% with a focus on photomechanics"
% F. Sur, B. Blaysat, M. Grédiac
% 2020

s=7;  
t=2*s+1;  % subset size
[x0,y0] = meshgrid(1:200,1:51);


% to reproduce Figures 2, 3, 4 of the paper, uncomment the corresponding lines:

% displacement a)  (Figure 2)
case_disp='a';
amplitude_def=.2;
def=amplitude_def*ones(51,200);

% displacement b)  (Figure 3)
% case_disp='b';
% amplitude_def=.2;
% period_def=90;
% def = amplitude_def*sin(2*pi*x0/period_def);

% displacement c)  (Figure 4)
% case_disp='c';
% amplitude_def=.2;
% period_def=10;
% def = amplitude_def*sin(2*pi*x0/period_def);

% interpolation 1)
interpscheme='exact';

% interpolation 2)
% interpscheme='linear';

% interpolation 3)
% interpscheme='cubic';


% render synthetic speckle images
gamma = 0.9; 
p = 80; 
m=25;
Ima_ref = 128*(1+gamma*sin(2*pi*(x0+def).^1.5/p^1.5));
Ima_def = 128*(1+gamma*sin(2*pi*x0.^1.5/p^1.5));


% calculate displacements
[ retrieved_displacement, retrieved_displacement_correc, predicted_displacement, sgf ] = calculate_displacements_1D(Ima_ref,Ima_def,s,def,interpscheme,case_disp);


% plot figures

figure, plot((s+1):size(Ima_ref,2)-(s+1),retrieved_displacement(m,(s+1):size(Ima_ref,2)-(s+1))-def(m,(s+1):size(Ima_ref,2)-(s+1)),...
    (s+1):size(Ima_ref,2)-(s+1),retrieved_displacement_correc(m,(s+1):size(Ima_ref,2)-(s+1))-def(m,(s+1):size(Ima_ref,2)-(s+1)),...
    (s+1):size(Ima_ref,2)-(s+1),predicted_displacement(m,(s+1):size(Ima_ref,2)-(s+1))-def(m,(s+1):size(Ima_ref,2)-(s+1)),'--',...
    (s+1):size(Ima_ref,2)-(s+1),sgf(m,(s+1):size(Ima_ref,2)-(s+1))-def(m,(s+1):size(Ima_ref,2)-(s+1)),'--','LineWidth',3)
lgd=legend('retrieved bias (SSD)','retrieved bias ($\widetilde{\mathrm{SSD}}$)','predicted bias','predicted bias with SG');
set(lgd,'fontsize',11,'Interpreter','latex')
grid on
xlabel('x')
if (strcmp(case_disp,'a')), axis([0,200,-0.01,0.01]); end
if (strcmp(case_disp,'b')), axis([0,200,-0.1,0.1]); end
if (strcmp(case_disp,'c')), axis([0,200,-0.4,0.4]); end

figure, h=plot(1:200,retrieved_displacement(m,:),1:200,retrieved_displacement_correc(m,:),...
    1:200,predicted_displacement(m,:),'--',1:200,sgf(t,:),'--',1:200,def(m,:),'LineWidth',3);
set(h(5),'LineWidth',1)
lgd=legend('retrieved with SSD','retrieved with $\widetilde{\mathrm{SSD}}$','predicted', 'output of SG filter','GT displacement');
set(lgd,'fontsize',11,'Interpreter','latex')
grid on
xlabel('x')
if (strcmp(case_disp,'a')), axis([0,200,0.17,0.23]); end
if (strcmp(case_disp,'b')), axis([0,200,-0.2,0.2]); end
if (strcmp(case_disp,'c')), axis([0,200,-0.2,0.2]); end

