clear all, close all
% script for reproducing the experiments in Section 4.2 of:
% "On biases in displacement estimation for image registration, 
% with a focus on photomechanics"
% F. Sur, B. Blaysat, M. Grédiac
% 2020


% Change the values of the following parameters:

% s: half-size of the subset

% order: order of the shape function, cf Section 1
% (0: constant, 1: linear, 2: quadratic)      

% option: modified SSD criterion, cf Section 4.2.5
% ('NO': standard SSD, 'GT': Eq. 4.5, '1D': Eq. 4.6, '2D': Eq. 4.7) 

% Img00 and Img01: input images
% speckle images (Sections 4.2.2 and 4.2.3) generated by BSpeckleRender2:
% https://members.loria.fr/FSur/software/BSpeckleRender/
% checkerboard images (Section 4.2.4) generated by:
% script_render_ckb.m in the present archive

% kappa: regularization parameter in modified SSD criteria (Section 4.2.5)
% 1e-2 with large speckle, 1e-4 with fine speckle
% not used with option='NO'
kappa = 0;

% output directory where images are saved:
dirname='images/';


% to reproduce the figures of the paper, uncomment the corresponding lines:

% Figure 8:
s = 3
order = 0
option = 'NO'
Img00 = double(imread('specklelarge01.png'))/2^12;
Img01 = double(imread('specklelarge00.png'))/2^12;

% Figure 9:
% s = 6
% order = 1
% option = 'NO'
% Img00 = double(imread('specklelarge01.png'))/2^12;
% Img01 = double(imread('specklelarge00.png'))/2^12;

% Figures 10 and 11:
% s = 9
% order = 2
% option = 'NO'
% Img00 = double(imread('specklelarge01.png'))/2^12;
% Img01 = double(imread('specklelarge00.png'))/2^12;

% Figure 12:
% s = 3
% order = 0
% option = 'NO'
% Img00 = double(imread('specklefine01.png'))/2^12;
% Img01 = double(imread('specklefine00.png'))/2^12;

% Figure 13:
% s = 6
% order = 1
% option = 'NO'
% Img00 = double(imread('specklefine01.png'))/2^12;
% Img01 = double(imread('specklefine00.png'))/2^12;

% Figures 14 and 15:
% s = 9
% order = 2
% option = 'NO'
% Img00 = double(imread('specklefine01.png'))/2^12;
% Img01 = double(imread('specklefine00.png'))/2^12;

% Figure 17:
% s = 3
% order = 0
% option = 'NO'
% Img00 = double(imread('checkerboard01.png'))/2^12;
% Img01 = double(imread('checkerboard00.png'))/2^12;

% Figure 18:
% s = 6
% order = 1
% option = 'NO'
% Img00 = double(imread('checkerboard01.png'))/2^12;
% Img01 = double(imread('checkerboard00.png'))/2^12;

% Figure 19:
% s = 9
% order = 0
% option = 'GT'
% Img00 = double(imread('specklelarge01.png'))/2^12;
% Img01 = double(imread('specklelarge00.png'))/2^12;
% kappa=1e-4

% Figure 20:
% s = 9
% order = 0
% option = 'GT'
% Img00 = double(imread('specklefine01.png'))/2^12;
% Img01 = double(imread('specklefine00.png'))/2^12;
% kappa=1e-2

% Figure 21: 
% s = 9
% order = 0
% option = '1D'
% Img00 = double(imread('specklelarge01.png'))/2^12;
% Img01 = double(imread('specklelarge00.png'))/2^12;
% kappa=1e-4

% Figure 22:
% s = 9
% order = 1
% option = '1D'
% Img00 = double(imread('specklefine01.png'))/2^12;
% Img01 = double(imread('specklefine00.png'))/2^12;
% kappa=1e-2

% Figure 23:
% s = 9
% order = 0
% option = '2D'
% Img00 = double(imread('specklelarge01.png'))/2^12;
% Img01 = double(imread('specklelarge00.png'))/2^12;
% kappa=1e-4

compute_U(s,order,option,Img00,Img01,dirname,kappa)

